@@ -1,6 +1,11 @@
 /* Define if C++ compiler supports covariant virtual methods. */
 #define HAVE_COVARIANT_RETURNS 1
 
+/* Define to 1 if C++ compiler supports nullptr keyword. */
+#if _MSC_VER >= 1600
+# define HAVE_NULLPTR 1
+#endif
+
 /* Define to 1 if you have an STL implementation that supports useful string
    specialization. */
 #define HAVE_GOOD_STL 1
