@@ -1,5 +1,5 @@
 /*
-*  Copyright 2001-2009 Internet2
+*  Copyright 2001-2010 Internet2
  * 
 * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -41,18 +41,18 @@ namespace xmltooling {
         virtual ~AbstractXMLObjectMarshaller();
 
         xercesc::DOMElement* marshall(
-            xercesc::DOMDocument* document=NULL
+            xercesc::DOMDocument* document=nullptr
 #ifndef XMLTOOLING_NO_XMLSEC
-            ,const std::vector<xmlsignature::Signature*>* sigs=NULL
-            ,const Credential* credential=NULL
+            ,const std::vector<xmlsignature::Signature*>* sigs=nullptr
+            ,const Credential* credential=nullptr
 #endif
             ) const;
 
         xercesc::DOMElement* marshall(
             xercesc::DOMElement* parentElement
 #ifndef XMLTOOLING_NO_XMLSEC
-            ,const std::vector<xmlsignature::Signature*>* sigs=NULL
-            ,const Credential* credential=NULL
+            ,const std::vector<xmlsignature::Signature*>* sigs=nullptr
+            ,const Credential* credential=nullptr
 #endif
             ) const;
         
@@ -81,7 +81,7 @@ namespace xmltooling {
          * @throws SignatureException thrown if a problem occurs during signature creation 
          */
         void marshallInto(
-            xercesc::DOMElement* targetElement, const std::vector<xmlsignature::Signature*>* sigs, const Credential* credential=NULL
+            xercesc::DOMElement* targetElement, const std::vector<xmlsignature::Signature*>* sigs, const Credential* credential=nullptr
             ) const;
 #else
         /**
@@ -141,6 +141,12 @@ namespace xmltooling {
          * @throws MarshallingException thrown if there is a problem marshalling an attribute
          */
         virtual void marshallAttributes(xercesc::DOMElement* domElement) const;
+
+        /**
+         * Called before marshalling in the event that a new DOM is being generated.
+         * <p>Allows objects to adjust internal state prior to the marshalling step.
+         */
+        virtual void prepareForMarshalling() const;
     };
     
 };
