@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2009 Internet2
+ *  Copyright 2001-2010 Internet2
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -44,7 +44,7 @@ namespace xmltooling {
          *
          * @param keyInfo   surrounding KeyInfo context object
          */
-        KeyInfoCredentialContext(const xmlsignature::KeyInfo* keyInfo=NULL);
+        KeyInfoCredentialContext(const xmlsignature::KeyInfo* keyInfo=nullptr);
 
         /**
          * Constructor
