@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2009 Internet2
+ *  Copyright 2001-2010 Internet2
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -39,17 +39,9 @@ namespace xmltooling {
         /**
          * Constructor.
          * 
-         * If a DOM is supplied, the following XML content is supported:
-         * 
-         * <ul>
-         *  <li>&lt;KeyInfoResolver&gt; elements with a type attribute
-         * </ul>
-         * 
-         * XML namespaces are ignored in the processing of this content.
-         * 
          * @param e DOM to supply configuration for provider
          */
-        OpenSSLTrustEngine(const xercesc::DOMElement* e=NULL);
+        OpenSSLTrustEngine(const xercesc::DOMElement* e=nullptr);
         
     public:
         virtual ~OpenSSLTrustEngine();
@@ -77,7 +69,7 @@ namespace xmltooling {
         virtual bool validate(
             X509* certEE, STACK_OF(X509)* certChain,
             const CredentialResolver& credResolver,
-            CredentialCriteria* criteria=NULL
+            CredentialCriteria* criteria=nullptr
             ) const=0;
     };
     
