@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2009 Internet2
+ *  Copyright 2001-2010 Internet2
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -51,7 +51,7 @@ namespace xmltooling {
          * 
          * @param e DOM to supply configuration for provider
          */
-        TrustEngine(const xercesc::DOMElement* e=NULL);
+        TrustEngine(const xercesc::DOMElement* e=nullptr);
         
         /** Custom KeyInfoResolver instance. */
         KeyInfoResolver* m_keyInfoResolver;
