@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2009 Internet2
+ *  Copyright 2001-2011 Internet2
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@
 #include "exceptions.h"
 #include "logging.h"
 #include "security/ChainingTrustEngine.h"
+#include "security/CredentialCriteria.h"
 #include "util/XMLHelper.h"
 
 #include <algorithm>
@@ -48,23 +49,13 @@ static const XMLCh type[] =                         UNICODE_LITERAL_4(t,y,p,e);
 
 ChainingTrustEngine::ChainingTrustEngine(const DOMElement* e) : TrustEngine(e) {
     Category& log=Category::getInstance(XMLTOOLING_LOGCAT".TrustEngine."CHAINING_TRUSTENGINE);
-    e = e ? XMLHelper::getFirstChildElement(e, _TrustEngine) : NULL;
+    e = e ? XMLHelper::getFirstChildElement(e, _TrustEngine) : nullptr;
     while (e) {
         try {
-            auto_ptr_char temp(e->getAttributeNS(NULL,type));
-            if (temp.get() && *temp.get()) {
-                log.info("building TrustEngine of type %s", temp.get());
-                TrustEngine* engine = XMLToolingConfig::getConfig().TrustEngineManager.newPlugin(temp.get(), e);
-                m_engines.push_back(engine);
-                SignatureTrustEngine* sig = dynamic_cast<SignatureTrustEngine*>(engine);
-                if (sig)
-                    m_sigEngines.push_back(sig);
-                X509TrustEngine* x509 = dynamic_cast<X509TrustEngine*>(engine);
-                if (x509)
-                    m_x509Engines.push_back(x509);
-                OpenSSLTrustEngine* ossl = dynamic_cast<OpenSSLTrustEngine*>(engine);
-                if (ossl)
-                    m_osslEngines.push_back(ossl);
+            string t = XMLHelper::getAttrString(e, nullptr, type);
+            if (!t.empty()) {
+                log.info("building TrustEngine of type %s", t.c_str());
+                addTrustEngine(XMLToolingConfig::getConfig().TrustEngineManager.newPlugin(t.c_str(), e));
             }
         }
         catch (exception& ex) {
@@ -81,24 +72,59 @@ ChainingTrustEngine::~ChainingTrustEngine() {
 void ChainingTrustEngine::addTrustEngine(TrustEngine* newEngine)
 {
     m_engines.push_back(newEngine);
+    SignatureTrustEngine* sig = dynamic_cast<SignatureTrustEngine*>(newEngine);
+    if (sig)
+        m_sigEngines.push_back(sig);
+    X509TrustEngine* x509 = dynamic_cast<X509TrustEngine*>(newEngine);
+    if (x509)
+        m_x509Engines.push_back(x509);
+    OpenSSLTrustEngine* ossl = dynamic_cast<OpenSSLTrustEngine*>(newEngine);
+    if (ossl)
+        m_osslEngines.push_back(ossl);
 }
 
 TrustEngine* ChainingTrustEngine::removeTrustEngine(TrustEngine* oldEngine)
 {
-    for (vector<TrustEngine*>::iterator i=m_engines.begin(); i!=m_engines.end(); i++) {
-        if (oldEngine==(*i)) {
-            m_engines.erase(i);
-            return oldEngine;
+    vector<TrustEngine*>::iterator i = find(m_engines.begin(), m_engines.end(), oldEngine);
+    if (i != m_engines.end()) {
+        m_engines.erase(i);
+
+        SignatureTrustEngine* sig = dynamic_cast<SignatureTrustEngine*>(oldEngine);
+        if (sig) {
+            vector<SignatureTrustEngine*>::iterator s = find(m_sigEngines.begin(), m_sigEngines.end(), sig);
+            if (s != m_sigEngines.end())
+                m_sigEngines.erase(s);
         }
+
+        X509TrustEngine* x509 = dynamic_cast<X509TrustEngine*>(oldEngine);
+        if (x509) {
+            vector<X509TrustEngine*>::iterator x = find(m_x509Engines.begin(), m_x509Engines.end(), x509);
+            if (x != m_x509Engines.end())
+                m_x509Engines.erase(x);
+        }
+
+        OpenSSLTrustEngine* ossl = dynamic_cast<OpenSSLTrustEngine*>(oldEngine);
+        if (ossl) {
+            vector<OpenSSLTrustEngine*>::iterator o = find(m_osslEngines.begin(), m_osslEngines.end(), ossl);
+            if (o != m_osslEngines.end())
+                m_osslEngines.erase(o);
+        }
+
+        return oldEngine;
     }
-    return NULL;
+    return nullptr;
 }
 
 bool ChainingTrustEngine::validate(Signature& sig, const CredentialResolver& credResolver, CredentialCriteria* criteria) const
 {
+    unsigned int usage = criteria ? criteria->getUsage() : 0;
     for (vector<SignatureTrustEngine*>::const_iterator i=m_sigEngines.begin(); i!=m_sigEngines.end(); ++i) {
         if ((*i)->validate(sig,credResolver,criteria))
             return true;
+        if (criteria) {
+            criteria->reset();
+            criteria->setUsage(usage);
+        }
     }
     return false;
 }
@@ -113,9 +139,14 @@ bool ChainingTrustEngine::validate(
     CredentialCriteria* criteria
     ) const
 {
+    unsigned int usage = criteria ? criteria->getUsage() : 0;
     for (vector<SignatureTrustEngine*>::const_iterator i=m_sigEngines.begin(); i!=m_sigEngines.end(); ++i) {
         if ((*i)->validate(sigAlgorithm, sig, keyInfo, in, in_len, credResolver, criteria))
             return true;
+        if (criteria) {
+            criteria->reset();
+            criteria->setUsage(usage);
+        }
     }
     return false;
 }
@@ -127,9 +158,14 @@ bool ChainingTrustEngine::validate(
     CredentialCriteria* criteria
     ) const
 {
+    unsigned int usage = criteria ? criteria->getUsage() : 0;
     for (vector<X509TrustEngine*>::const_iterator i=m_x509Engines.begin(); i!=m_x509Engines.end(); ++i) {
         if ((*i)->validate(certEE,certChain,credResolver,criteria))
             return true;
+        if (criteria) {
+            criteria->reset();
+            criteria->setUsage(usage);
+        }
     }
     return false;
 }
@@ -141,9 +177,14 @@ bool ChainingTrustEngine::validate(
     CredentialCriteria* criteria
     ) const
 {
+    unsigned int usage = criteria ? criteria->getUsage() : 0;
     for (vector<OpenSSLTrustEngine*>::const_iterator i=m_osslEngines.begin(); i!=m_osslEngines.end(); ++i) {
         if ((*i)->validate(certEE,certChain,credResolver,criteria))
             return true;
+        if (criteria) {
+            criteria->reset();
+            criteria->setUsage(usage);
+        }
     }
     return false;
 }
