@@ -1,5 +1,5 @@
 /*
- *  Copyright 2001-2009 Internet2
+ *  Copyright 2001-2010 Internet2
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -46,7 +46,7 @@ namespace xmlsignature {
          * 
          * @param key the key to use
          */
-        SignatureValidator(XSECCryptoKey* key=NULL);
+        SignatureValidator(XSECCryptoKey* key=nullptr);
 
         /**
          * Constructor using a Credential
